# Generated by Django 2.2.13 on 2020-10-19 22:17

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("grapple", "0003_stubmodel"),
    ]

    operations = [
        migrations.DeleteModel(
            name="StubModel",
        ),
    ]
